@@ -1,34 +1,34 @@
 
 --- Unix implementation of filesystem and platform abstractions.
-
-local assert, type, table, io, package, math, os, ipairs =
-      assert, type, table, io, package, math, os, ipairs
-
-module("luarocks.fs.unix", package.seeall)
+local unix = {}
 
 local fs = require("luarocks.fs")
 
 local cfg = require("luarocks.cfg")
 local dir = require("luarocks.dir")
-local fs = require("luarocks.fs")
 local util = require("luarocks.util")
 
-math.randomseed(os.time())
-
 --- Annotate command string for quiet execution.
 -- @param cmd string: A command-line string.
 -- @return string: The command-line, with silencing annotation.
-function quiet(cmd)
+function unix.quiet(cmd)
    return cmd.." 1> /dev/null 2> /dev/null"
 end
 
+--- Annotate command string for execution with quiet stderr.
+-- @param cmd string: A command-line string.
+-- @return string: The command-line, with stderr silencing annotation.
+function unix.quiet_stderr(cmd)
+   return cmd.." 2> /dev/null"
+end
+
 --- Return an absolute pathname from a potentially relative one.
 -- @param pathname string: pathname to convert.
 -- @param relative_to string or nil: path to prepend when making
 -- pathname absolute, or the current dir in the dir stack if
 -- not given.
 -- @return string: The pathname converted to absolute.
-function absolute_name(pathname, relative_to)
+function unix.absolute_name(pathname, relative_to)
    assert(type(pathname) == "string")
    assert(type(relative_to) == "string" or not relative_to)
 
@@ -40,6 +40,14 @@ function absolute_name(pathname, relative_to)
    end
 end
 
+--- Return the root directory for the given path.
+-- In Unix, root is always "/".
+-- @param pathname string: pathname to use.
+-- @return string: The root of the given pathname.
+function unix.root_of(_)
+   return "/"
+end
+
 --- Create a wrapper to make a script executable from the command-line.
 -- @param file string: Pathname of script to be made executable.
 -- @param dest string: Directory where to put the wrapper.
@@ -47,7 +55,7 @@ end
 -- @param version string: rock version to be used in loader context.
 -- @return boolean or (nil, string): True if succeeded, or nil and
 -- an error message.
-function wrap_script(file, dest, name, version)
+function unix.wrap_script(file, dest, name, version)
    assert(type(file) == "string")
    assert(type(dest) == "string")
    
@@ -64,7 +72,7 @@ function wrap_script(file, dest, name, version)
    local addctx = "local k,l,_=pcall(require,"..util.LQ("luarocks.loader")..") _=k and l.add_context("..util.LQ(name)..","..util.LQ(version)..")"
    wrapper:write('exec '..fs.Q(lua)..' -e '..fs.Q(ppaths)..' -e '..fs.Q(addctx)..' '..fs.Q(file)..' "$@"\n')
    wrapper:close()
-   if fs.chmod(wrapname, "0755") then
+   if fs.chmod(wrapname, cfg.perm_exec) then
       return true
    else
       return nil, "Could not make "..wrapname.." executable."
@@ -76,7 +84,7 @@ end
 -- @param filename string: the file name with full path.
 -- @return boolean: returns true if file is an actual binary
 -- (or if it couldn't check) or false if it is a Lua wrapper.
-function is_actual_binary(filename)
+function unix.is_actual_binary(filename)
    if filename:match("%.lua$") then
       return false
    end
@@ -93,8 +101,8 @@ function is_actual_binary(filename)
    return first ~= "#!"
 end
 
-function copy_binary(filename, dest) 
-   return fs.copy(filename, dest, "0755")
+function unix.copy_binary(filename, dest) 
+   return fs.copy(filename, dest, cfg.perm_exec)
 end
 
 --- Move a file on top of the other.
@@ -107,6 +115,12 @@ end
 -- which will replace old_file.
 -- @return boolean or (nil, string): True if succeeded, or nil and
 -- an error message.
-function replace_file(old_file, new_file)
+function unix.replace_file(old_file, new_file)
    return os.rename(new_file, old_file)
 end
+
+function unix.tmpname()
+   return os.tmpname()
+end
+
+return unix
