@@ -106,13 +106,22 @@ end
 
 -- An ode to the multitude of JSON libraries out there...
 local function require_json()
-   for _, lib in ipairs({ "cjson", "dkjson", "json" }) do
+   local list = { "cjson", "dkjson", "json" }
+   for _, lib in ipairs(list) do
       local json_ok, json = pcall(require, lib)
       if json_ok then
          return json_ok, json
       end
    end
-   return nil
+   local errmsg = "Failed loading "
+   for i, name in ipairs(list) do
+      if i == #list then
+         errmsg = errmsg .."and '"..name.."'. Use 'luarocks search <partial-name>' to search for a library and 'luarocks install <name>' to install one."
+      else
+         errmsg = errmsg .."'"..name.."', "
+      end
+   end
+   return nil, errmsg
 end
 
 local function redact_api_url(url)
@@ -126,12 +135,12 @@ if not ltn12_ok then -- If not using LuaSocket and/or LuaSec...
 function Api:request(url, params, post_params)
    local vars = cfg.variables
    local json_ok, json = require_json()
-   if not json_ok then return nil, "A JSON library is required for this command." end
+   if not json_ok then return nil, "A JSON library is required for this command. "..json end
    
    if cfg.downloader == "wget" then
-      local curl_ok = fs.execute_quiet(vars.CURL, "--version")
+      local curl_ok, err = fs.is_tool_available(vars.CURL, "curl")
       if not curl_ok then
-         return nil, "Missing network helper program 'curl'.\nMake sure 'curl' is installed and available from your path."
+         return nil, err
       end
    end
 
@@ -192,7 +201,7 @@ local warned_luasec = false
 
 function Api:request(url, params, post_params)
    local json_ok, json = require_json()
-   if not json_ok then return nil, "A JSON library is required for this command." end
+   if not json_ok then return nil, "A JSON library is required for this command. "..json end
    local server = tostring(self.config.server)
    local http_ok, http
    local via = "luasocket"
